(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('react-konva')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'react-konva'], factory) :
  (global = global || self, factory(global.reactKonvaPortal = {}, global.react, global.reactKonva));
}(this, (function (exports, React, reactKonva) {
  var React__default = 'default' in React ? React['default'] : React;

  var DEFAULT_Z_INDEX = 0;
  function zIndexComparator(a, b) {
    var _a$zIndex = a.zIndex,
        zIndexA = _a$zIndex === void 0 ? DEFAULT_Z_INDEX : _a$zIndex;
    var _b$zIndex = b.zIndex,
        zIndexB = _b$zIndex === void 0 ? DEFAULT_Z_INDEX : _b$zIndex;
    if (zIndexA === zIndexB) return 0;
    return zIndexA > zIndexB ? 1 : -1;
  }

  function PortalManagerComponent(_, ref) {
    var _useState = React.useState([]),
        portals = _useState[0],
        setPortals = _useState[1];

    React.useImperativeHandle(ref, function () {
      return {
        mount: function mount(key, children, zIndex) {
          return setPortals(function (currValue) {
            return [].concat(currValue, [{
              key: key,
              children: children,
              zIndex: zIndex
            }]).sort(zIndexComparator);
          });
        },
        update: function update(key, children, zIndex) {
          return setPortals(function (prevValue) {
            var zIndexChanged = false;
            var currValue = prevValue.map(function (item) {
              if (item.key !== key) return item;
              zIndexChanged = item.zIndex !== zIndex;
              return {
                key: key,
                children: children,
                zIndex: zIndex
              };
            });
            return zIndexChanged ? currValue.sort(zIndexComparator) : currValue;
          });
        },
        unmount: function unmount(key) {
          return setPortals(function (prevValue) {
            return prevValue.filter(function (item) {
              return item.key !== key;
            });
          });
        }
      };
    }, [setPortals]);
    return portals.map(function (_ref) {
      var key = _ref.key,
          children = _ref.children;
      return React__default.createElement(React__default.Fragment, {
        key: key
      }, children);
    });
  }

  var PortalManager = React.forwardRef(PortalManagerComponent);

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  function pendingPredicate(key) {
    return function (item) {
      return key === item.key && (item.type === 'mount' || item.type === 'update');
    };
  }
  var PortalStageContext = React.createContext(null);

  function PortalStageContextProvider(_ref) {
    var mount = _ref.mount,
        unmount = _ref.unmount,
        update = _ref.update,
        children = _ref.children;
    var contextValue = React.useMemo(function () {
      return {
        mount: mount,
        unmount: unmount,
        update: update
      };
    }, [mount, unmount, update]);
    return React__default.createElement(PortalStageContext.Provider, {
      value: contextValue
    }, children);
  }

  function PortalStage(_ref2) {
    var children = _ref2.children,
        portalLayerProps = _ref2.portalLayerProps,
        stageProps = _objectWithoutPropertiesLoose(_ref2, ["children", "portalLayerProps"]);

    var seqRef = React.useRef(0);
    var queueRef = React.useRef([]);
    var managerRef = React.useRef(null);
    React.useLayoutEffect(function () {
      var handlers = {
        mount: function mount(o) {
          var _managerRef$current;

          if (o.type === 'mount') (_managerRef$current = managerRef.current) === null || _managerRef$current === void 0 ? void 0 : _managerRef$current.mount(o.key, o.children, o.zIndex);
        },
        update: function update(op) {
          var _managerRef$current2;

          if (op.type === 'update') (_managerRef$current2 = managerRef.current) === null || _managerRef$current2 === void 0 ? void 0 : _managerRef$current2.update(op.key, op.children, op.zIndex);
        },
        unmount: function unmount(op) {
          var _managerRef$current3;

          if (op.type === 'unmount') (_managerRef$current3 = managerRef.current) === null || _managerRef$current3 === void 0 ? void 0 : _managerRef$current3.unmount(op.key);
        }
      };

      while (queueRef.current.length) {
        var action = queueRef.current.pop();
        if (action) handlers[action.type].call(null, action);
      }
    }, []);
    var mount = React.useCallback(function (children, zIndex) {
      var key = ++seqRef.current;
      if (managerRef.current) managerRef.current.mount(key, children, zIndex);else queueRef.current.push({
        type: 'mount',
        key: key,
        children: children,
        zIndex: zIndex
      });
      return key;
    }, []);
    var update = React.useCallback(function (key, children, zIndex) {
      if (managerRef.current) managerRef.current.update(key, children, zIndex);else {
        var index = queueRef.current.findIndex(pendingPredicate(key));
        if (index === -1) queueRef.current.push({
          type: 'mount',
          key: key,
          children: children,
          zIndex: zIndex
        });else queueRef.current[index] = {
          type: 'mount',
          key: key,
          children: children,
          zIndex: zIndex
        };
      }
    }, []);
    var unmount = React.useCallback(function (key) {
      if (managerRef.current) managerRef.current.unmount(key);else queueRef.current.push({
        type: 'unmount',
        key: key
      });
    }, []);
    return React__default.createElement(reactKonva.Stage, Object.assign({}, stageProps), React__default.createElement(PortalStageContextProvider, {
      mount: mount,
      update: update,
      unmount: unmount
    }, children, React__default.createElement(reactKonva.Layer, Object.assign({}, portalLayerProps), React__default.createElement(PortalManager, {
      ref: managerRef
    }))));
  }

  var PortalPhase;

  (function (PortalPhase) {
    PortalPhase[PortalPhase["INITIAL"] = 0] = "INITIAL";
    PortalPhase[PortalPhase["WILL_MOUNT"] = 1] = "WILL_MOUNT";
    PortalPhase[PortalPhase["DID_MOUNT"] = 2] = "DID_MOUNT";
    PortalPhase[PortalPhase["WILL_UNMOUNT"] = 3] = "WILL_UNMOUNT";
  })(PortalPhase || (PortalPhase = {}));

  var ERROR_MESSAGE = 'You should wrap your root component with <Stage /> from "react-konva-portal" in order for Portals to work';
  function isPortalManager(val) {
    return val && typeof val === 'object' && typeof val.mount === 'function' && typeof val.update === 'function' && typeof val.unmount === 'function';
  }
  function assertPortalManager(val) {
    if (!isPortalManager(val)) throw new Error(ERROR_MESSAGE);
    return true;
  }
  function Portal(_ref) {
    var children = _ref.children,
        _ref$zIndex = _ref.zIndex,
        zIndex = _ref$zIndex === void 0 ? DEFAULT_Z_INDEX : _ref$zIndex;
    var keyRef = React.useRef(-1);
    var phaseRef = React.useRef(PortalPhase.INITIAL);
    var stagePortals = React.useContext(PortalStageContext);
    React.useLayoutEffect(function () {
      assertPortalManager(stagePortals);
    }, [stagePortals]);
    React.useLayoutEffect(function () {
      return function () {
        phaseRef.current = PortalPhase.WILL_UNMOUNT;
      };
    }, []);
    var mountAsync = React.useCallback(function () {
      try {
        return Promise.resolve(Promise.resolve()).then(function () {
          keyRef.current = stagePortals === null || stagePortals === void 0 ? void 0 : stagePortals.mount(children, zIndex);
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }, []);
    React.useLayoutEffect(function () {
      if (phaseRef.current === PortalPhase.INITIAL) {
        phaseRef.current = PortalPhase.WILL_MOUNT;
        mountAsync().then(function () {
          phaseRef.current = PortalPhase.DID_MOUNT;
        });
      }

      if (phaseRef.current === PortalPhase.DID_MOUNT) {
        stagePortals === null || stagePortals === void 0 ? void 0 : stagePortals.update(keyRef.current, children, zIndex);
      }

      return function () {
        if (phaseRef.current === PortalPhase.WILL_UNMOUNT) {
          stagePortals === null || stagePortals === void 0 ? void 0 : stagePortals.unmount(keyRef.current);
        }
      };
    }, [children, zIndex, mountAsync, stagePortals]);
    return null;
  }

  exports.Portal = Portal;
  exports.Stage = PortalStage;

})));
//# sourceMappingURL=index.umd.js.map
